]> Pileus Git - ~andy/fetchmail/blobdiff - rfc822.c
Compute FQDN when Kerberos is in use.
[~andy/fetchmail] / rfc822.c
index 5e9e2da56a63bb8690262af1d0ee2a56dc68a684..9b574b55023c839d031d9ace7cf8f8eaf27ba720 100644 (file)
--- a/rfc822.c
+++ b/rfc822.c
 
 #ifdef TESTMAIN
 static int verbose;
+char *program_name = "rfc822";
 #endif /* TESTMAIN */
 
-void reply_hack(buf, host)
+char *reply_hack(buf, host)
 /* hack message headers so replies will work properly */
 char *buf;             /* header to be hacked */
 const char *host;      /* server hostname */
 {
-    char *from, *cp;
+    char *from, *cp, last_nws = '\0', *parens_from = NULL;
     int parendepth, state, has_bare_name_part, has_host_part;
+    int addresscount = 1;
 
     if (strncasecmp("From: ", buf, 6)
        && strncasecmp("To: ", buf, 4)
        && strncasecmp("Reply-To: ", buf, 10)
        && strncasecmp("Return-Path: ", buf, 13)
        && strncasecmp("Cc: ", buf, 4)
-       && strncasecmp("Bcc: ", buf, 5)) {
-       return;
+       && strncasecmp("Bcc: ", buf, 5)
+       && strncasecmp("Resent-From: ", buf, 13)
+       && strncasecmp("Resent-To: ", buf, 11)
+       && strncasecmp("Resent-Cc: ", buf, 11)
+       && strncasecmp("Resent-Bcc: ", buf, 12)
+       && strncasecmp("Apparently-From:", buf, 16)
+       && strncasecmp("Apparently-To:", buf, 14)
+       && strncasecmp("Sender:", buf, 7)
+       && strncasecmp("Resent-Sender:", buf, 14)
+       ) {
+       return(buf);
     }
 
+#ifndef TESTMAIN
+    /* make room to hack the address; buf must be malloced */
+    for (cp = buf; *cp; cp++)
+       if (*cp == ',' || isspace(*cp))
+           addresscount++;
+    buf = (char *)xrealloc(buf, strlen(buf) + addresscount * strlen(host) + 1);
+#endif /* TESTMAIN */
+
+    /*
+     * This is going to foo up on some ill-formed addresses.
+     * Note that we don't rewrite the fake address <> in order to
+     * avoid screwing up bounce suppression with a null Return-Path.
+     */
+
     parendepth = state = 0;
     has_host_part = has_bare_name_part = FALSE;
     for (from = buf; *from; from++)
@@ -63,6 +88,8 @@ const char *host;     /* server hostname */
                break;
 
            case 1:     /* we've seen the colon, we're looking for addresses */
+               if (!isspace(*from))
+                   last_nws = *from;
                if (*from == '<')
                    state = 3;
                else if (*from == '@')
@@ -70,17 +97,21 @@ const char *host;   /* server hostname */
                else if (*from == '"')
                    state = 2;
                /*
* Not expanding on from[-1] == ';' deals with groupnames,
                 * an obscure misfeature described in sections
                 * 6.1, 6.2.6, and A.1.5 of the RFC822 standard.
                 */
-               else if ((*from == ',' || HEADER_END(from) || from[1] == '(')
+               else if ((*from == ',' || HEADER_END(from))
                         && has_bare_name_part
                         && !has_host_part
-                        && from[-1] != ';')
+                        && last_nws != ';')
                {
                    int hostlen;
+                   char *p;
 
+                   p = from;
+                   if (parens_from)
+                       from = parens_from;
                    while (isspace(*from) || (*from == ','))
                        --from;
                    from++;
@@ -89,9 +120,16 @@ const char *host;   /* server hostname */
                        cp[hostlen+1] = *cp;
                    *from++ = '@';
                    memcpy(from, host, hostlen);
-                   from += hostlen;
+                   from = p + hostlen + 1;
                    has_host_part = TRUE;
                } 
+               else if (from[1] == '('
+                        && has_bare_name_part
+                        && !has_host_part
+                        && last_nws != ';' && last_nws != ')')
+               {
+                   parens_from = from;
+               } 
                else if (!isspace(*from))
                    has_bare_name_part = TRUE;
                break;
@@ -104,7 +142,7 @@ const char *host;   /* server hostname */
            case 3:     /* we're in a <>-enclosed address */
                if (*from == '@')
                    has_host_part = TRUE;
-               else if (*from == '>')
+               else if (*from == '>' && from[-1] != '<')
                {
                    state = 1;
                    if (!has_host_part)
@@ -128,8 +166,11 @@ const char *host;  /* server hostname */
         */
        if (from[-1] == ',' && !parendepth) {
          has_host_part = has_bare_name_part = FALSE;
+         parens_from = NULL;
        }
     }
+
+    return(buf);
 }
 
 char *nxtaddr(hdr)
@@ -70,17 +97,21 @@ const char *host;   /* server hostname */
                    state = 2;
                /*
-                * Not expanding on from[-1] == ';' deals with groupnames,
+                * Not expanding on last non-WS == ';' deals with groupnames,
                 * an obscure misfeature described in sections
                 * 6.1, 6.2.6, and A.1.5 of the RFC822 standard.